""" Module to access the Status endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, List, Union

from pydantic import BaseModel

from ...models import (
    PostUserRecentCustomStatusDeleteJsonBody,
    RemoveRecentCustomStatusJsonBody,
    Status,
    UpdateUserCustomStatusJsonBody,
    UpdateUserStatusJsonBody,
)
from ..base import ApiBaseClass


class StatusApi(ApiBaseClass):
    """Endpoints for getting and updating user statuses."""

    async def get_user_status(
        self,
        user_id: str,
    ) -> Status:
        """Get user status

        Get user status by id from the server.

        Permissions:
            Must be authenticated.

        Api Reference:
            `GetUserStatus <https://api.mattermost.com/#operation/GetUserStatus>`_
        """

        url = f"/users/{user_id}/status"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Status.parse_obj(response.json())

            return response200
        return response

    async def update_user_status(
        self,
        user_id: str,
        *,
        json_body: Union[UpdateUserStatusJsonBody, Dict],
    ) -> Status:
        """Update user status

        Manually set a user's status. When setting a user's status, the status
        will remain that value until set \"online\" again, which will return the
        status to being automatically updated based on user activity.

        Permissions:
            Must have `edit_other_users` permission for the team.

        Api Reference:
            `UpdateUserStatus <https://api.mattermost.com/#operation/UpdateUserStatus>`_
        """

        url = f"/users/{user_id}/status"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Status.parse_obj(response.json())

            return response200
        return response

    async def get_users_statuses_by_ids(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> List[Status]:
        """Get user statuses by id

        Get a list of user statuses by id from the server.

        Permissions:
            Must be authenticated.

        Api Reference:
            `GetUsersStatusesByIds <https://api.mattermost.com/#operation/GetUsersStatusesByIds>`_
        """

        url = "/users/status/ids"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Status.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def update_user_custom_status(
        self,
        user_id: str,
        *,
        json_body: Union[UpdateUserCustomStatusJsonBody, Dict],
    ) -> None:
        """Update user custom status

        Updates a user's custom status by setting the value in the user's props
        and updates the user. Also save the given custom status to the recent
        custom statuses in the user's props

        Permissions:
            Must be logged in as the user whose custom status is being
            updated.

        Api Reference:
            `UpdateUserCustomStatus <https://api.mattermost.com/#operation/UpdateUserCustomStatus>`_
        """

        url = f"/users/{user_id}/status/custom"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def unset_user_custom_status(
        self,
        user_id: str,
    ) -> None:
        """Unsets user custom status

        Unsets a user's custom status by updating the user's props and updates
        the user

        Permissions:
            Must be logged in as the user whose custom status is being
            removed.

        Api Reference:
            `UnsetUserCustomStatus <https://api.mattermost.com/#operation/UnsetUserCustomStatus>`_
        """

        url = f"/users/{user_id}/status/custom"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def remove_recent_custom_status(
        self,
        user_id: str,
        *,
        json_body: Union[RemoveRecentCustomStatusJsonBody, Dict],
    ) -> None:
        """Delete user's recent custom status

        Deletes a user's recent custom status by removing the specific status
        from the recentCustomStatuses in the user's props and updates the user.

        Permissions:
            Must be logged in as the user whose recent custom status is
            being deleted.

        Api Reference:
            `RemoveRecentCustomStatus <https://api.mattermost.com/#operation/RemoveRecentCustomStatus>`_
        """

        url = f"/users/{user_id}/status/custom/recent"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def post_user_recent_custom_status_delete(
        self,
        user_id: str,
        *,
        json_body: Union[PostUserRecentCustomStatusDeleteJsonBody, Dict],
    ) -> None:
        """Delete user's recent custom status

        Deletes a user's recent custom status by removing the specific status
        from the recentCustomStatuses in the user's props and updates the user.

        Permissions:
            Must be logged in as the user whose recent custom status is
            being deleted.

        Api Reference:
            `PostUserRecentCustomStatusDelete <https://api.mattermost.com/#operation/PostUserRecentCustomStatusDelete>`_
        """

        url = f"/users/{user_id}/status/custom/recent/delete"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response



""" Module to access the Uploads endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from pydantic import BaseModel

from ...models import CreateUploadJsonBody, FileInfo, UploadDataFormData, UploadSession
from ..base import ApiBaseClass


class UploadsApi(ApiBaseClass):
    """Endpoints for creating and performing file uploads."""

    async def create_upload(
        self,
        *,
        json_body: Union[CreateUploadJsonBody, Dict],
    ) -> UploadSession:
        """Create an upload

        Creates a new upload session.

        Permissions:
            Must have `upload_file` permission.
        Minimum Server Version:
            5.28

        Api Reference:
            `CreateUpload <https://api.mattermost.com/#operation/CreateUpload>`_
        """

        url = "/uploads"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = UploadSession.parse_obj(response.json())

            return response201
        return response

    async def get_upload(
        self,
        upload_id: str,
    ) -> None:
        """Get an upload session

        Gets an upload session that has been previously created.

        Permissions:
            Must be logged in as the user who created the upload
            session.

        Api Reference:
            `GetUpload <https://api.mattermost.com/#operation/GetUpload>`_
        """

        url = f"/uploads/{upload_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    async def upload_data(
        self,
        upload_id: str,
        *,
        form_data: UploadDataFormData,
    ) -> Union[FileInfo, None]:
        """Perform a file upload

        Starts or resumes a file upload.
        To resume an existing (incomplete) upload, data should be sent starting
        from the offset specified in the upload session object.

        The request body can be in one of two formats:
        - Binary file content streamed in request's body
        - multipart/form-data

        Permissions:
            Must be logged in as the user who created the upload
            session.

        Api Reference:
            `UploadData <https://api.mattermost.com/#operation/UploadData>`_
        """

        url = f"/uploads/{upload_id}"

        request_kwargs = {
            "url": url,
            "data": form_data,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = FileInfo.parse_obj(response.json())

            return response201
        return response



""" Module to access the Channels endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union, cast

from pydantic import BaseModel

from ...models import (
    AddChannelMemberJsonBody,
    Channel,
    ChannelListWithTeamData,
    ChannelMember,
    ChannelModeration,
    ChannelModerationPatch,
    ChannelNotifyProps,
    ChannelStats,
    ChannelUnread,
    CreateChannelJsonBody,
    MoveChannelJsonBody,
    OrderedSidebarCategories,
    PatchChannelJsonBody,
    PostList,
    SearchAllChannelsJsonBody,
    SearchAllChannelsResponse200,
    SearchArchivedChannelsJsonBody,
    SearchChannelsJsonBody,
    SearchGroupChannelsJsonBody,
    SidebarCategory,
    StatusOK,
    UpdateChannelJsonBody,
    UpdateChannelMemberSchemeRolesJsonBody,
    UpdateChannelPrivacyJsonBody,
    UpdateChannelRolesJsonBody,
    UpdateChannelSchemeJsonBody,
    ViewChannelJsonBody,
    ViewChannelResponse200,
)
from ..base import ApiBaseClass


class ChannelsApi(ApiBaseClass):
    """Endpoints for creating, getting and interacting with channels."""

    def get_all_channels(
        self,
        *,
        not_associated_to_group: Optional[str] = None,
        page: Optional[int] = 0,
        per_page: Optional[int] = 0,
        exclude_default_channels: Optional[bool] = False,
        include_deleted: Optional[bool] = False,
        include_total_count: Optional[bool] = False,
        exclude_policy_constrained: Optional[bool] = False,
    ) -> ChannelListWithTeamData:
        """Get a list of all channels



        Permissions:
            `manage_system`

        Api Reference:
            `GetAllChannels <https://api.mattermost.com/#operation/GetAllChannels>`_
        """

        url = "/channels"
        params: Dict[str, Any] = {
            "not_associated_to_group": not_associated_to_group,
            "page": page,
            "per_page": per_page,
            "exclude_default_channels": exclude_default_channels,
            "include_deleted": include_deleted,
            "include_total_count": include_total_count,
            "exclude_policy_constrained": exclude_policy_constrained,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelListWithTeamData.parse_obj(response.json())

            return response200
        return response

    def create_channel(
        self,
        *,
        json_body: Union[CreateChannelJsonBody, Dict],
    ) -> Channel:
        """Create a channel

        Create a new channel.

        Permissions:
            If creating a public channel, `create_public_channel`
            permission is required. If creating a private channel,
            `create_private_channel` permission is required.

        Api Reference:
            `CreateChannel <https://api.mattermost.com/#operation/CreateChannel>`_
        """

        url = "/channels"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Channel.parse_obj(response.json())

            return response201
        return response

    def create_direct_channel(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> Channel:
        """Create a direct message channel

        Create a new direct message channel between two users.

        Permissions:
            Must be one of the two users and have
            `create_direct_channel` permission. Having the
            `manage_system` permission voids the previous requirements.

        Api Reference:
            `CreateDirectChannel <https://api.mattermost.com/#operation/CreateDirectChannel>`_
        """

        url = "/channels/direct"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Channel.parse_obj(response.json())

            return response201
        return response

    def create_group_channel(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> Channel:
        """Create a group message channel

        Create a new group message channel to group of users. If the logged in
        user's id is not included in the list, it will be appended to the end.

        Permissions:
            Must have `create_group_channel` permission.

        Api Reference:
            `CreateGroupChannel <https://api.mattermost.com/#operation/CreateGroupChannel>`_
        """

        url = "/channels/group"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Channel.parse_obj(response.json())

            return response201
        return response

    def search_all_channels(
        self,
        *,
        json_body: Union[SearchAllChannelsJsonBody, Dict],
        system_console: Optional[bool] = True,
    ) -> SearchAllChannelsResponse200:
        """Search all private and open type channels across all teams

        Returns all private and open type channels where 'term' matches on the
        name, display name, or purpose of
        the channel.

        Configured 'default' channels (ex Town Square and Off-Topic) can be
        excluded from the results
        with the `exclude_default_channels` boolean parameter.

        Channels that are associated (via GroupChannel records) to a given group
        can be excluded from the results
        with the `not_associated_to_group` parameter and a group id string.

        Api Reference:
            `SearchAllChannels <https://api.mattermost.com/#operation/SearchAllChannels>`_
        """

        url = "/channels/search"
        params: Dict[str, Any] = {
            "system_console": system_console,
        }
        params = {k: v for k, v in params.items() if v is not None}

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SearchAllChannelsResponse200.parse_obj(response.json())

            return response200
        return response

    def search_group_channels(
        self,
        *,
        json_body: Union[SearchGroupChannelsJsonBody, Dict],
    ) -> List[Channel]:
        """Search Group Channels

        Get a list of group channels for a user which members' usernames match
        the search term.

        Minimum Server Version:
            5.14

        Api Reference:
            `SearchGroupChannels <https://api.mattermost.com/#operation/SearchGroupChannels>`_
        """

        url = "/channels/group/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_public_channels_by_ids_for_team(
        self,
        team_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> List[Channel]:
        """Get a list of channels by ids

        Get a list of public channels on a team by id.

        Permissions:
            `view_team` for the team the channels are on.

        Api Reference:
            `GetPublicChannelsByIdsForTeam <https://api.mattermost.com/#operation/GetPublicChannelsByIdsForTeam>`_
        """

        url = f"/teams/{team_id}/channels/ids"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channel_members_timezones(
        self,
        channel_id: str,
    ) -> List[str]:
        """Get timezones in a channel

        Get a list of timezones for the users who are in this channel.

        Permissions:
            Must have the `read_channel` permission.
        Minimum Server Version:
            5.6

        Api Reference:
            `GetChannelMembersTimezones <https://api.mattermost.com/#operation/GetChannelMembersTimezones>`_
        """

        url = f"/channels/{channel_id}/timezones"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response

    def get_channel(
        self,
        channel_id: str,
    ) -> Channel:
        """Get a channel

        Get channel from the provided channel id string.

        Permissions:
            `read_channel` permission for the channel.

        Api Reference:
            `GetChannel <https://api.mattermost.com/#operation/GetChannel>`_
        """

        url = f"/channels/{channel_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def update_channel(
        self,
        channel_id: str,
        *,
        json_body: Union[UpdateChannelJsonBody, Dict],
    ) -> Channel:
        """Update a channel

        Update a channel. The fields that can be updated are listed as
        parameters. Omitted fields will be treated as blanks.

        Permissions:
            If updating a public channel,
            `manage_public_channel_members` permission is required. If
            updating a private channel, `manage_private_channel_members`
            permission is required.

        Api Reference:
            `UpdateChannel <https://api.mattermost.com/#operation/UpdateChannel>`_
        """

        url = f"/channels/{channel_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def delete_channel(
        self,
        channel_id: str,
    ) -> StatusOK:
        """Delete a channel

        Archives a channel. This will set the `deleteAt` to the current
        timestamp in the database. Soft deleted channels may not be accessible
        in the user interface. They can be viewed and unarchived in the **System
        Console > User Management > Channels** based on your license. Direct and
        group message channels cannot be deleted.

        As of server version 5.28, optionally use the `permanent=true` query
        parameter to permanently delete the channel for compliance reasons. To
        use this feature `ServiceSettings.EnableAPIChannelDeletion` must be set
        to `true` in the server's configuration.  If you permanently delete a
        channel this action is not recoverable outside of a database backup.

        `delete_private_channel` permission if the channel is private,
        or have `manage_system` permission.

        Permissions:
            `delete_public_channel` permission if the channel is public,

        Api Reference:
            `DeleteChannel <https://api.mattermost.com/#operation/DeleteChannel>`_
        """

        url = f"/channels/{channel_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def patch_channel(
        self,
        channel_id: str,
        *,
        json_body: Union[PatchChannelJsonBody, Dict],
    ) -> Channel:
        """Patch a channel

        Partially update a channel by providing only the fields you want to
        update. Omitted fields will not be updated. The fields that can be
        updated are defined in the request body, all other provided fields will
        be ignored.

        Permissions:
            If updating a public channel,
            `manage_public_channel_members` permission is required. If
            updating a private channel, `manage_private_channel_members`
            permission is required.

        Api Reference:
            `PatchChannel <https://api.mattermost.com/#operation/PatchChannel>`_
        """

        url = f"/channels/{channel_id}/patch"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def update_channel_privacy(
        self,
        channel_id: str,
        *,
        json_body: Union[UpdateChannelPrivacyJsonBody, Dict],
    ) -> Channel:
        """Update channel's privacy

        Updates channel's privacy allowing changing a channel from Public to
        Private and back.

        Permissions:
            `manage_team` permission for the channels team on version <
            5.28. `convert_public_channel_to_private` permission for the
            channel if updating privacy to 'P' on version >= 5.28.
            `convert_private_channel_to_public` permission for the
            channel if updating privacy to 'O' on version >= 5.28.
        Minimum Server Version:
            5.16

        Api Reference:
            `UpdateChannelPrivacy <https://api.mattermost.com/#operation/UpdateChannelPrivacy>`_
        """

        url = f"/channels/{channel_id}/privacy"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def restore_channel(
        self,
        channel_id: str,
    ) -> Channel:
        """Restore a channel

        Restore channel from the provided channel id string.

        Permissions:
            `manage_team` permission for the team of the channel.
        Minimum Server Version:
            3.10

        Api Reference:
            `RestoreChannel <https://api.mattermost.com/#operation/RestoreChannel>`_
        """

        url = f"/channels/{channel_id}/restore"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def move_channel(
        self,
        channel_id: str,
        *,
        json_body: Union[MoveChannelJsonBody, Dict],
    ) -> Channel:
        """Move a channel

        Move a channel to another team.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `MoveChannel <https://api.mattermost.com/#operation/MoveChannel>`_
        """

        url = f"/channels/{channel_id}/move"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def get_channel_stats(
        self,
        channel_id: str,
    ) -> ChannelStats:
        """Get channel statistics

        Get statistics for a channel.

        Permissions:
            Must have the `read_channel` permission.

        Api Reference:
            `GetChannelStats <https://api.mattermost.com/#operation/GetChannelStats>`_
        """

        url = f"/channels/{channel_id}/stats"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelStats.parse_obj(response.json())

            return response200
        return response

    def get_pinned_posts(
        self,
        channel_id: str,
    ) -> PostList:
        """Get a channel's pinned posts

        Get a list of pinned posts for channel.

        Api Reference:
            `GetPinnedPosts <https://api.mattermost.com/#operation/GetPinnedPosts>`_
        """

        url = f"/channels/{channel_id}/pinned"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostList.parse_obj(response.json())

            return response200
        return response

    def get_public_channels_for_team(
        self,
        team_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Channel]:
        """Get public channels

        Get a page of public channels on a team based on query string parameters
        - page and per_page.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.

        Api Reference:
            `GetPublicChannelsForTeam <https://api.mattermost.com/#operation/GetPublicChannelsForTeam>`_
        """

        url = f"/teams/{team_id}/channels"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_private_channels_for_team(
        self,
        team_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Channel]:
        """Get private channels

        Get a page of private channels on a team based on query string
        parameters - team_id, page and per_page.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetPrivateChannelsForTeam <https://api.mattermost.com/#operation/GetPrivateChannelsForTeam>`_
        """

        url = f"/teams/{team_id}/channels/private"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_deleted_channels_for_team(
        self,
        team_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Channel]:
        """Get deleted channels

        Get a page of deleted channels on a team based on query string
        parameters - team_id, page and per_page.

        Minimum Server Version:
            3.10

        Api Reference:
            `GetDeletedChannelsForTeam <https://api.mattermost.com/#operation/GetDeletedChannelsForTeam>`_
        """

        url = f"/teams/{team_id}/channels/deleted"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def autocomplete_channels_for_team(
        self,
        team_id: str,
        *,
        name: str,
    ) -> List[Channel]:
        """Autocomplete channels

        Autocomplete public channels on a team based on the search term provided
        in the request URL.

        Permissions:
            Must have the `list_team_channels` permission.
        Minimum Server Version:
            4.7

        Api Reference:
            `AutocompleteChannelsForTeam <https://api.mattermost.com/#operation/AutocompleteChannelsForTeam>`_
        """

        url = f"/teams/{team_id}/channels/autocomplete"
        params: Dict[str, Any] = {
            "name": name,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def autocomplete_channels_for_team_for_search(
        self,
        team_id: str,
        *,
        name: str,
    ) -> List[Channel]:
        """Autocomplete channels for search

        Autocomplete your channels on a team based on the search term provided
        in the request URL.

        Permissions:
            Must have the `list_team_channels` permission.
        Minimum Server Version:
            5.4

        Api Reference:
            `AutocompleteChannelsForTeamForSearch <https://api.mattermost.com/#operation/AutocompleteChannelsForTeamForSearch>`_
        """

        url = f"/teams/{team_id}/channels/search_autocomplete"
        params: Dict[str, Any] = {
            "name": name,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def search_channels(
        self,
        team_id: str,
        *,
        json_body: Union[SearchChannelsJsonBody, Dict],
    ) -> List[Channel]:
        """Search channels

        Search public channels on a team based on the search term provided in
        the request body.

        In server version 5.16 and later, a user without the
        `list_team_channels` permission will be able to use this endpoint, with
        the search results limited to the channels that the user is a member of.

        Permissions:
            Must have the `list_team_channels` permission.

        Api Reference:
            `SearchChannels <https://api.mattermost.com/#operation/SearchChannels>`_
        """

        url = f"/teams/{team_id}/channels/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = []
            _response201 = response.json()
            for response201_item_data in _response201:
                response201_item = Channel.parse_obj(response201_item_data)

                response201.append(response201_item)

            return response201
        return response

    def search_archived_channels(
        self,
        team_id: str,
        *,
        json_body: Union[SearchArchivedChannelsJsonBody, Dict],
    ) -> List[Channel]:
        """Search archived channels

        Search archived channels on a team based on the search term provided in
        the request body.

        In server version 5.18 and later, a user without the
        `list_team_channels` permission will be able to use this endpoint, with
        the search results limited to the channels that the user is a member of.

        Permissions:
            Must have the `list_team_channels` permission.
        Minimum Server Version:
            5.18

        Api Reference:
            `SearchArchivedChannels <https://api.mattermost.com/#operation/SearchArchivedChannels>`_
        """

        url = f"/teams/{team_id}/channels/search_archived"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = []
            _response201 = response.json()
            for response201_item_data in _response201:
                response201_item = Channel.parse_obj(response201_item_data)

                response201.append(response201_item)

            return response201
        return response

    def get_channel_by_name(
        self,
        team_id: str,
        channel_name: str,
        *,
        include_deleted: Optional[bool] = False,
    ) -> Channel:
        """Get a channel by name

        Gets channel from the provided team id and channel name strings.

        Permissions:
            `read_channel` permission for the channel.

        Api Reference:
            `GetChannelByName <https://api.mattermost.com/#operation/GetChannelByName>`_
        """

        url = f"/teams/{team_id}/channels/name/{channel_name}"
        params: Dict[str, Any] = {
            "include_deleted": include_deleted,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def get_channel_by_name_for_team_name(
        self,
        team_name: str,
        channel_name: str,
        *,
        include_deleted: Optional[bool] = False,
    ) -> Channel:
        """Get a channel by name and team name

        Gets a channel from the provided team name and channel name strings.

        Permissions:
            `read_channel` permission for the channel.

        Api Reference:
            `GetChannelByNameForTeamName <https://api.mattermost.com/#operation/GetChannelByNameForTeamName>`_
        """

        url = f"/teams/name/{team_name}/channels/name/{channel_name}"
        params: Dict[str, Any] = {
            "include_deleted": include_deleted,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Channel.parse_obj(response.json())

            return response200
        return response

    def get_channel_members(
        self,
        channel_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[ChannelMember]:
        """Get channel members

        Get a page of members for a channel.

        Permissions:
            `read_channel` permission for the channel.

        Api Reference:
            `GetChannelMembers <https://api.mattermost.com/#operation/GetChannelMembers>`_
        """

        url = f"/channels/{channel_id}/members"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ChannelMember.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def add_channel_member(
        self,
        channel_id: str,
        *,
        json_body: Union[AddChannelMemberJsonBody, Dict],
    ) -> ChannelMember:
        """Add user to channel

        Add a user to a channel by creating a channel member object.

        Api Reference:
            `AddChannelMember <https://api.mattermost.com/#operation/AddChannelMember>`_
        """

        url = f"/channels/{channel_id}/members"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = ChannelMember.parse_obj(response.json())

            return response201
        return response

    def get_channel_members_by_ids(
        self,
        channel_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> List[ChannelMember]:
        """Get channel members by ids

        Get a list of channel members based on the provided user ids.

        Permissions:
            Must have the `read_channel` permission.

        Api Reference:
            `GetChannelMembersByIds <https://api.mattermost.com/#operation/GetChannelMembersByIds>`_
        """

        url = f"/channels/{channel_id}/members/ids"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ChannelMember.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channel_member(
        self,
        channel_id: str,
        user_id: str,
    ) -> ChannelMember:
        """Get channel member

        Get a channel member.

        Permissions:
            `read_channel` permission for the channel.

        Api Reference:
            `GetChannelMember <https://api.mattermost.com/#operation/GetChannelMember>`_
        """

        url = f"/channels/{channel_id}/members/{user_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelMember.parse_obj(response.json())

            return response200
        return response

    def remove_user_from_channel(
        self,
        channel_id: str,
        user_id: str,
    ) -> StatusOK:
        """Remove user from channel

        Delete a channel member, effectively removing them from a channel.

        In server version 5.3 and later, channel members can only be deleted
        from public or private channels.
        `manage_private_channel_members` permission if the channel is private.

        Permissions:
            `manage_public_channel_members` permission if the channel is
            public.

        Api Reference:
            `RemoveUserFromChannel <https://api.mattermost.com/#operation/RemoveUserFromChannel>`_
        """

        url = f"/channels/{channel_id}/members/{user_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def update_channel_roles(
        self,
        channel_id: str,
        user_id: str,
        *,
        json_body: Union[UpdateChannelRolesJsonBody, Dict],
    ) -> StatusOK:
        """Update channel roles

        Update a user's roles for a channel.

        Permissions:
            Must have `manage_channel_roles` permission for the channel.

        Api Reference:
            `UpdateChannelRoles <https://api.mattermost.com/#operation/UpdateChannelRoles>`_
        """

        url = f"/channels/{channel_id}/members/{user_id}/roles"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def update_channel_member_scheme_roles(
        self,
        channel_id: str,
        user_id: str,
        *,
        json_body: Union[UpdateChannelMemberSchemeRolesJsonBody, Dict],
    ) -> StatusOK:
        """Update the scheme-derived roles of a channel member.

        Update a channel member's scheme_admin/scheme_user properties. Typically
        this should either be `scheme_admin=false, scheme_user=true` for
        ordinary channel member, or `scheme_admin=true, scheme_user=true` for a
        channel admin.

        Permissions:
            Must be authenticated and have the `manage_channel_roles`
            permission.
        Minimum Server Version:
            5.0

        Api Reference:
            `UpdateChannelMemberSchemeRoles <https://api.mattermost.com/#operation/UpdateChannelMemberSchemeRoles>`_
        """

        url = f"/channels/{channel_id}/members/{user_id}/schemeRoles"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def update_channel_notify_props(
        self,
        channel_id: str,
        user_id: str,
        *,
        json_body: Union[ChannelNotifyProps, Dict],
    ) -> StatusOK:
        """Update channel notifications

        Update a user's notification properties for a channel. Only the provided
        fields are updated.

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.

        Api Reference:
            `UpdateChannelNotifyProps <https://api.mattermost.com/#operation/UpdateChannelNotifyProps>`_
        """

        url = f"/channels/{channel_id}/members/{user_id}/notify_props"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def view_channel(
        self,
        user_id: str,
        *,
        json_body: Union[ViewChannelJsonBody, Dict],
    ) -> ViewChannelResponse200:
        """View channel

        Perform all the actions involved in viewing a channel. This includes
        marking channels as read, clearing push notifications, and updating the
        active channel.

        __Response only includes `last_viewed_at_times` in Mattermost server 4.3
        and newer.__

        Permissions:
            Must be logged in as user or have `edit_other_users`
            permission.

        Api Reference:
            `ViewChannel <https://api.mattermost.com/#operation/ViewChannel>`_
        """

        url = f"/channels/members/{user_id}/view"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ViewChannelResponse200.parse_obj(response.json())

            return response200
        return response

    def get_channel_members_for_user(
        self,
        user_id: str,
        team_id: str,
    ) -> List[ChannelMember]:
        """Get channel memberships and roles for a user

        Get all channel memberships and associated membership roles (i.e.
        `channel_user`, `channel_admin`) for a user on a specific team.

        Permissions:
            Logged in as the user and `view_team` permission for the
            team. Having `manage_system` permission voids the previous
            requirements.

        Api Reference:
            `GetChannelMembersForUser <https://api.mattermost.com/#operation/GetChannelMembersForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/members"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ChannelMember.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channels_for_team_for_user(
        self,
        user_id: str,
        team_id: str,
        *,
        include_deleted: Optional[bool] = False,
        last_delete_at: Optional[int] = 0,
    ) -> List[Channel]:
        """Get channels for user

        Get all the channels on a team for a user.

        Permissions:
            Logged in as the user, or have `edit_other_users`
            permission, and `view_team` permission for the team.

        Api Reference:
            `GetChannelsForTeamForUser <https://api.mattermost.com/#operation/GetChannelsForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels"
        params: Dict[str, Any] = {
            "include_deleted": include_deleted,
            "last_delete_at": last_delete_at,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channels_for_user(
        self,
        user_id: str,
        *,
        last_delete_at: Optional[int] = 0,
        include_deleted: Optional[bool] = False,
    ) -> List[Channel]:
        """Get all channels from all teams

        Get all channels from all teams that a user is a member of.

        Permissions:
            Logged in as the user, or have `edit_other_users`
            permission.
        Minimum Server Version:
            6.1

        Api Reference:
            `GetChannelsForUser <https://api.mattermost.com/#operation/GetChannelsForUser>`_
        """

        url = f"/users/{user_id}/channels"
        params: Dict[str, Any] = {
            "last_delete_at": last_delete_at,
            "include_deleted": include_deleted,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channel_unread(
        self,
        user_id: str,
        channel_id: str,
    ) -> ChannelUnread:
        """Get unread messages

        Get the total unread messages and mentions for a channel for a user.

        Permissions:
            Must be logged in as user and have the `read_channel`
            permission, or have `edit_other_usrs` permission.

        Api Reference:
            `GetChannelUnread <https://api.mattermost.com/#operation/GetChannelUnread>`_
        """

        url = f"/users/{user_id}/channels/{channel_id}/unread"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelUnread.parse_obj(response.json())

            return response200
        return response

    def update_channel_scheme(
        self,
        channel_id: str,
        *,
        json_body: Union[UpdateChannelSchemeJsonBody, Dict],
    ) -> StatusOK:
        """Set a channel's scheme

        Set a channel's scheme, more specifically sets the scheme_id value of a
        channel record.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            4.10

        Api Reference:
            `UpdateChannelScheme <https://api.mattermost.com/#operation/UpdateChannelScheme>`_
        """

        url = f"/channels/{channel_id}/scheme"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def channel_members_minus_group_members(
        self,
        channel_id: str,
        *,
        group_ids: str = "",
        page: Optional[int] = 0,
        per_page: Optional[int] = 0,
    ) -> None:
        """Channel members minus group members.

        Get the set of users who are members of the channel minus the set of
        users who are members of the given groups.
        Each user object contains an array of group objects representing the
        group memberships for that user.
        Each user object contains the boolean fields `scheme_guest`,
        `scheme_user`, and `scheme_admin` representing the roles that user has
        for the given channel.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.14

        Api Reference:
            `ChannelMembersMinusGroupMembers <https://api.mattermost.com/#operation/ChannelMembersMinusGroupMembers>`_
        """

        url = f"/channels/{channel_id}/members_minus_group_members"
        params: Dict[str, Any] = {
            "group_ids": group_ids,
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def get_channel_member_counts_by_group(
        self,
        channel_id: str,
        *,
        include_timezones: Optional[bool] = False,
    ) -> None:
        """Channel members counts for each group that has atleast one member in the
        channel

        Returns a set of ChannelMemberCountByGroup objects which contain a
        `group_id`, `channel_member_count` and a
        `channel_member_timezones_count`.

        Permissions:
            Must have `read_channel` permission for the given channel.
        Minimum Server Version:
            5.24

        Api Reference:
            `GetChannelMemberCountsByGroup <https://api.mattermost.com/#operation/GetChannelMemberCountsByGroup>`_
        """

        url = f"/channels/{channel_id}/member_counts_by_group"
        params: Dict[str, Any] = {
            "include_timezones": include_timezones,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def get_channel_moderations(
        self,
        channel_id: str,
    ) -> List[ChannelModeration]:
        """Get information about channel's moderation.



        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.22

        Api Reference:
            `GetChannelModerations <https://api.mattermost.com/#operation/GetChannelModerations>`_
        """

        url = f"/channels/{channel_id}/moderations"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ChannelModeration.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def patch_channel_moderations(
        self,
        channel_id: str,
        *,
        json_body: Union[ChannelModerationPatch, Dict],
    ) -> List[ChannelModeration]:
        """Update a channel's moderation settings.



        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.22

        Api Reference:
            `PatchChannelModerations <https://api.mattermost.com/#operation/PatchChannelModerations>`_
        """

        url = f"/channels/{channel_id}/moderations/patch"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ChannelModeration.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_sidebar_categories_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
    ) -> List[OrderedSidebarCategories]:
        """Get user's sidebar categories

        Get a list of sidebar categories that will appear in the user's sidebar
        on the given team, including a list of channel IDs in each category.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetSidebarCategoriesForTeamForUser <https://api.mattermost.com/#operation/GetSidebarCategoriesForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = OrderedSidebarCategories.parse_obj(
                    response200_item_data
                )

                response200.append(response200_item)

            return response200
        return response

    def update_sidebar_categories_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        *,
        json_body: Union[List[SidebarCategory], Dict],
    ) -> SidebarCategory:
        """Update user's sidebar categories

        Update any number of sidebar categories for the user on the given team.
        This can be used to reorder the channels in these categories.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `UpdateSidebarCategoriesForTeamForUser <https://api.mattermost.com/#operation/UpdateSidebarCategoriesForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories"
        json_json_body = []
        for json_body_item_data in json_body:

            if isinstance(json_body_item_data, BaseModel):
                json_body_item = json_body_item_data.dict(exclude_unset=True)
            else:
                json_body_item = json_body_item_data

            json_json_body.append(json_body_item)

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SidebarCategory.parse_obj(response.json())

            return response200
        return response

    def create_sidebar_category_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        *,
        json_body: Union[SidebarCategory, Dict],
    ) -> SidebarCategory:
        """Create user's sidebar category

        Create a custom sidebar category for the user on the given team.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `CreateSidebarCategoryForTeamForUser <https://api.mattermost.com/#operation/CreateSidebarCategoryForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SidebarCategory.parse_obj(response.json())

            return response200
        return response

    def get_sidebar_category_order_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
    ) -> List[str]:
        """Get user's sidebar category order

        Returns the order of the sidebar categories for a user on the given team
        as an array of IDs.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetSidebarCategoryOrderForTeamForUser <https://api.mattermost.com/#operation/GetSidebarCategoryOrderForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories/order"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response

    def update_sidebar_category_order_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        *,
        json_body: Union[List[str], Dict],
    ) -> List[str]:
        """Update user's sidebar category order

        Updates the order of the sidebar categories for a user on the given
        team. The provided array must include the IDs of all categories on the
        team.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `UpdateSidebarCategoryOrderForTeamForUser <https://api.mattermost.com/#operation/UpdateSidebarCategoryOrderForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories/order"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = cast(List[str], response.json())

            return response200
        return response

    def get_sidebar_category_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        category_id: str,
    ) -> SidebarCategory:
        """Get sidebar category

        Returns a single sidebar category for the user on the given team.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `GetSidebarCategoryForTeamForUser <https://api.mattermost.com/#operation/GetSidebarCategoryForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories/{category_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SidebarCategory.parse_obj(response.json())

            return response200
        return response

    def update_sidebar_category_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        category_id: str,
        *,
        json_body: Union[SidebarCategory, Dict],
    ) -> SidebarCategory:
        """Update sidebar category

        Updates a single sidebar category for the user on the given team.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `UpdateSidebarCategoryForTeamForUser <https://api.mattermost.com/#operation/UpdateSidebarCategoryForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories/{category_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SidebarCategory.parse_obj(response.json())

            return response200
        return response

    def remove_sidebar_category_for_team_for_user(
        self,
        team_id: str,
        user_id: str,
        category_id: str,
    ) -> SidebarCategory:
        """Delete sidebar category

        Deletes a single sidebar category for the user on the given team. Only
        custom categories can be deleted.

        Permissions:
            Must be authenticated and have the `list_team_channels`
            permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `RemoveSidebarCategoryForTeamForUser <https://api.mattermost.com/#operation/RemoveSidebarCategoryForTeamForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/channels/categories/{category_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = SidebarCategory.parse_obj(response.json())

            return response200
        return response



""" Contains all API endpoints """

__all__ = [
    "UsersApi",
    "BotsApi",
    "TermsOfServiceApi",
    "MigrateApi",
    "AuthenticationApi",
    "LdapApi",
    "SamlApi",
    "ThreadsApi",
    "DataRetentionApi",
    "StatusApi",
    "TeamsApi",
    "ChannelsApi",
    "PostsApi",
    "PreferencesApi",
    "FilesApi",
    "SearchApi",
    "UploadsApi",
    "JobsApi",
    "SystemApi",
    "RootApi",
    "EmojiApi",
    "WebhooksApi",
    "ComplianceApi",
    "GroupsApi",
    "ClusterApi",
    "BrandApi",
    "CommandsApi",
    "OauthApi",
    "ElasticsearchApi",
    "BleveApi",
    "PluginsApi",
    "RolesApi",
    "SchemesApi",
    "SharedChannelsApi",
    "OpenGraphApi",
    "ReactionsApi",
    "IntegrationActionsApi",
    "CloudApi",
    "PermissionsApi",
    "ImportsApi",
    "ExportsApi",
]

from .authentication import AuthenticationApi
from .bleve import BleveApi
from .bots import BotsApi
from .brand import BrandApi
from .channels import ChannelsApi
from .cloud import CloudApi
from .cluster import ClusterApi
from .commands import CommandsApi
from .compliance import ComplianceApi
from .data_retention import DataRetentionApi
from .elasticsearch import ElasticsearchApi
from .emoji import EmojiApi
from .exports import ExportsApi
from .files import FilesApi
from .groups import GroupsApi
from .imports import ImportsApi
from .integration_actions import IntegrationActionsApi
from .jobs import JobsApi
from .ldap import LdapApi
from .migrate import MigrateApi
from .oauth import OauthApi
from .open_graph import OpenGraphApi
from .permissions import PermissionsApi
from .plugins import PluginsApi
from .posts import PostsApi
from .preferences import PreferencesApi
from .reactions import ReactionsApi
from .roles import RolesApi
from .root import RootApi
from .saml import SamlApi
from .schemes import SchemesApi
from .search import SearchApi
from .shared_channels import SharedChannelsApi
from .status import StatusApi
from .system import SystemApi
from .teams import TeamsApi
from .terms_of_service import TermsOfServiceApi
from .threads import ThreadsApi
from .uploads import UploadsApi
from .users import UsersApi
from .webhooks import WebhooksApi



""" Module to access the IntegrationActions endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from pydantic import BaseModel

from ...models import (
    OpenInteractiveDialogJsonBody,
    StatusOK,
    SubmitInteractiveDialogJsonBody,
)
from ..base import ApiBaseClass


class IntegrationActionsApi(ApiBaseClass):
    """Endpoints for interactive actions for use by integrations."""

    def open_interactive_dialog(
        self,
        *,
        json_body: Union[OpenInteractiveDialogJsonBody, Dict],
    ) -> StatusOK:
        """Open a dialog

        Open an interactive dialog using a trigger ID provided by a slash
        command, or some other action payload. See
        https://docs.mattermost.com/developer/interactive-dialogs.html for more
        information on interactive dialogs.

        Minimum Server Version:
            5.6

        Api Reference:
            `OpenInteractiveDialog <https://api.mattermost.com/#operation/OpenInteractiveDialog>`_
        """

        url = "/actions/dialogs/open"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def submit_interactive_dialog(
        self,
        *,
        json_body: Union[SubmitInteractiveDialogJsonBody, Dict],
    ) -> StatusOK:
        """Submit a dialog

        Endpoint used by the Mattermost clients to submit a dialog. See
        https://docs.mattermost.com/developer/interactive-dialogs.html for more
        information on interactive dialogs.

        Minimum Server Version:
            5.6

        Api Reference:
            `SubmitInteractiveDialog <https://api.mattermost.com/#operation/SubmitInteractiveDialog>`_
        """

        url = "/actions/dialogs/submit"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Module to access the Brand endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from ...models import StatusOK, UploadBrandImageMultipartData
from ..base import ApiBaseClass


class BrandApi(ApiBaseClass):
    """Endpoints related to custom branding and white-labeling. See [our
    branding
    documentation](https://docs.mattermost.com/administration/branding.html)
    for more information."""

    def get_brand_image(
        self,
    ) -> str:
        """Get brand image

        Get the previously uploaded brand image. Returns 404 if no brand image
        has been uploaded.

        Permissions:
            No permission required.

        Api Reference:
            `GetBrandImage <https://api.mattermost.com/#operation/GetBrandImage>`_
        """

        url = "/brand/image"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = response.json()
            return response200
        return response

    def upload_brand_image(
        self,
        *,
        multipart_data: Union[UploadBrandImageMultipartData, Dict],
    ) -> StatusOK:
        """Upload brand image

        Uploads a brand image.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `UploadBrandImage <https://api.mattermost.com/#operation/UploadBrandImage>`_
        """

        url = "/brand/image"

        multipart_body_data = UploadBrandImageMultipartData.parse_obj(multipart_data)

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = StatusOK.parse_obj(response.json())

            return response201
        return response

    def delete_brand_image(
        self,
    ) -> StatusOK:
        """Delete current brand image

        Deletes the previously uploaded brand image. Returns 404 if no brand
        image has been uploaded.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.6

        Api Reference:
            `DeleteBrandImage <https://api.mattermost.com/#operation/DeleteBrandImage>`_
        """

        url = "/brand/image"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Base class and options for API clients """
# pylint: disable=no-self-use,too-few-public-methods

import abc
import asyncio
import inspect
import json
import logging
import socket
from typing import Any, Callable, Dict, Optional, Union
from urllib.parse import urljoin, urlparse

import httpx
import websockets.client as ws_client
import websockets.exceptions as ws_exceptions
from pydantic import AnyHttpUrl, AnyUrl, BaseModel, validator

logger = logging.getLogger("matterapi.client")
logger.setLevel(logging.INFO)


class HttpxClientOptions(BaseModel):
    """Options for the httpx client"""

    class Config:
        """Pydantic config for class"""

        extra = "allow"
        arbitrary_types_allowed = True

    timeout: Optional[Any] = 10.0
    """ Timeout for operations """
    verify: Union[bool, Any] = True
    """ If and how TLS should be verified. """
    cert: Optional[Any] = None
    proxies: Optional[Dict] = None
    auth: Optional[Any] = None


class AuthLogin(BaseModel):
    """Mattermost Auhentication Credentials"""

    login_id: str
    """ Login id for authentication """
    password: str
    """ Password """
    mfa_token: Optional[str]
    """ MultiFactorAuthentication token """


class AuthToken(httpx.Auth, BaseModel):
    """Mattermost Auhentication Token"""

    token: str
    """ Bearer Authentication token"""

    def auth_flow(self, request):
        request.headers["Authorization"] = f"Bearer {self.token}"
        yield request


class ApiClientOptions(BaseModel):
    """Options to be passed to a api client"""

    url: AnyHttpUrl
    """ Mattermost base url

        Example: https://localhost:8065
    """
    ws_url: Optional[AnyUrl]
    """ Mattermost websocket url """
    basepath: str = "/api/v4"
    """ API path. Usually you won't need to change this """
    auth: Optional[Union[AuthLogin, AuthToken]]
    """ Authentication information for the client

    Provide login information for tasks which require authentication.
    If set, this can be an instance of :class:`.AuthLogin` or :class:`.AuthToken`
    """
    httpx_client_options: Optional[HttpxClientOptions]
    """ Options to be passed to the underlying http client doing the actual requests """
    debug: bool = False
    """ Set to ``true`` to enable debugging """
    ws_concurrent: bool = True
    """ Change handling behaviour of websocket messages

    If set to true, websocket messages will be handled concurrently.
    This means that the loop will not wait until messages are handled
    but continue to wait/handle the next message. If set to false,
    messages will be handled sequentially. Handling of a new message
    only starts after the event_handler returns.
    """
    ws_reconnect_wait_time: int = 5
    """ The waiting time between re-connect attempts for websocket connections """
    skip_response_parsing: bool = False
    """ If this is set, responses will not be parsed into objects, but
        will be returned as raw httpx response """

    # pylint: disable=no-self-argument
    @validator("ws_url", pre=True, always=True)
    def _ws_url_setter(cls, value, values):
        """Set the websocket url from the url if not set explicitly"""
        if value is None:
            if "url" in values:
                url = urlparse(values["url"])
                scheme_map = {"https": "wss", "http": "ws"}
                url = url._replace(scheme=scheme_map[url.scheme])
                return url.geturl()
        else:
            return value

        raise ValueError(
            "'url' is invalid or not present in values and ws_url was not set explicitly"
        )


class BaseClient(BaseModel, metaclass=abc.ABCMeta):
    """
    Contains shared business logic of Sync and Async client for interacting with the mattermost api
    """

    options: Union[ApiClientOptions, Dict]
    """ Options to be passed to the client """
    active_token: Optional[str] = None
    """ The currently active authentication token.

    This will either be the token passed as :class:~AuthToken or the session token acquired
    through username/password based authentication.
    Used for websocket connections
    """

    # pylint: disable=no-self-argument
    @validator("options")
    def set_logging_mode(cls, value):
        if value.debug:
            logger.setLevel(logging.DEBUG)
        return value

    @abc.abstractmethod
    async def _login(self):
        pass

    async def _relogin(self):
        if inspect.iscoroutinefunction(self._login):
            await self._login()
        else:
            await asyncio.to_thread(self._login)

    async def start_ws(self, event_handler: Callable, relogin=False):
        """Start the websocket connection and pass messages to event_handler

        Connects to the websocket, completes authentication, and
        handles incoming messages. Will automatically try to reconnect
        to the server on connection errors.

        Args:
            event_handler: The message handler. Can be a function or coroutine.
                Get's the message passed as argument
            relogin: Set to ``True`` to run _login() after a connection error.
                This might be useful in case a session ends for username/password based logins
                and you need to acquire a new session token.
        """

        while True:
            try:
                await self._start_ws(event_handler)
            except (
                socket.gaierror,
                ws_exceptions.ConnectionClosedError,
                ConnectionRefusedError,
            ) as connection_error:
                logger.info(
                    (
                        "Connection to websocket closed. Either the server is not"
                        "reachable or authentication failed. Reconnecting in %s seconds"
                    ),
                    self.options.ws_reconnect_wait_time,
                    exc_info=True,
                )
                logger.info(connection_error)
            except ws_exceptions.WebSocketException:
                logger.exception("Got an unexpected exception from websocktes")

            await asyncio.sleep(self.options.ws_reconnect_wait_time)

            if relogin:
                await self._relogin()

    def start_ws_sync(self, event_handler: Callable, relogin=False):
        """Same as `.start_ws` but wraps the async call to be called synchronously"""
        asyncio.run(self.start_ws(event_handler, relogin))

    async def _handle_messages(self, event_handler: Callable, message: Dict):
        if inspect.iscoroutinefunction(event_handler):
            return asyncio.create_task(event_handler(message))
        return asyncio.create_task(asyncio.to_thread(event_handler, message))

    async def _handle_message_async(self, event_handler: Callable, websocket):
        async for raw_message in websocket:
            message = json.loads(raw_message)
            await asyncio.to_thread(event_handler, message)

    async def _start_ws(self, event_handler: Callable):
        json_auth_data = json.dumps(
            {
                "seq": 1,
                "action": "authentication_challenge",
                "data": {"token": self.active_token},
            }
        )

        logger.info("Connecting to websocket")
        ws_url = urljoin(self.options.ws_url, "api/v4/websocket")
        async with ws_client.connect(ws_url) as websocket:
            # Authenticate
            await websocket.send(json_auth_data)
            hello = json.loads(await websocket.recv())
            logger.debug("%s", hello)
            logger.info(
                "Connected to Mattermost server: %s", hello["data"]["server_version"]
            )
            # Wait for authentication 'OK'
            async for raw_message in websocket:
                message = json.loads(raw_message)
                logger.debug("%s", message)
                if message.get("seq_reply", None) == 1:
                    if message["status"] == "OK":
                        logger.info("Successfully authenticated to websocket")
                        break
                logger.info(
                    "Got a stray message during authentication procedure: %s", message
                )
            tasks = set()
            while True:
                try:
                    raw_message = await asyncio.wait_for(websocket.recv(), timeout=1.0)
                    message = json.loads(raw_message)
                    task = await self._handle_messages(event_handler, message)
                    if self.options.ws_concurrent:
                        tasks.add(task)
                    else:
                        await task
                except asyncio.TimeoutError:
                    ...
                # In case we handle tasks concurrently we will want to collect results/exceptions
                for task in list(tasks):
                    if task.done():
                        # pylint: disable=broad-except
                        try:
                            await task
                        except KeyboardInterrupt as kbdint:
                            raise kbdint
                        except Exception:
                            logger.exception("Websocket task failed")
                        tasks.remove(task)



""" Module to access the Webhooks endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    CreateIncomingWebhookJsonBody,
    CreateOutgoingWebhookJsonBody,
    IncomingWebhook,
    OutgoingWebhook,
    StatusOK,
    UpdateIncomingWebhookJsonBody,
    UpdateOutgoingWebhookJsonBody,
)
from ..base import ApiBaseClass


class WebhooksApi(ApiBaseClass):
    """Endpoints for creating, getting and updating webhooks."""

    def get_incoming_webhooks(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
        team_id: Optional[str] = None,
    ) -> List[IncomingWebhook]:
        """List incoming webhooks

        Get a page of a list of incoming webhooks. Optionally filter for a
        specific team using query parameters.

        Permissions:
            `manage_webhooks` for the system or `manage_webhooks` for
            the specific team.

        Api Reference:
            `GetIncomingWebhooks <https://api.mattermost.com/#operation/GetIncomingWebhooks>`_
        """

        url = "/hooks/incoming"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "team_id": team_id,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = IncomingWebhook.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def create_incoming_webhook(
        self,
        *,
        json_body: Union[CreateIncomingWebhookJsonBody, Dict],
    ) -> IncomingWebhook:
        """Create an incoming webhook

        Create an incoming webhook for a channel.

        `manage_others_incoming_webhooks` for the team the webhook is in if the
        user is different than the requester.

        Permissions:
            `manage_webhooks` for the team the webhook is in.

        Api Reference:
            `CreateIncomingWebhook <https://api.mattermost.com/#operation/CreateIncomingWebhook>`_
        """

        url = "/hooks/incoming"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = IncomingWebhook.parse_obj(response.json())

            return response201
        return response

    def get_incoming_webhook(
        self,
        hook_id: str,
    ) -> IncomingWebhook:
        """Get an incoming webhook

        Get an incoming webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `GetIncomingWebhook <https://api.mattermost.com/#operation/GetIncomingWebhook>`_
        """

        url = f"/hooks/incoming/{hook_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = IncomingWebhook.parse_obj(response.json())

            return response200
        return response

    def update_incoming_webhook(
        self,
        hook_id: str,
        *,
        json_body: Union[UpdateIncomingWebhookJsonBody, Dict],
    ) -> IncomingWebhook:
        """Update an incoming webhook

        Update an incoming webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `UpdateIncomingWebhook <https://api.mattermost.com/#operation/UpdateIncomingWebhook>`_
        """

        url = f"/hooks/incoming/{hook_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = IncomingWebhook.parse_obj(response.json())

            return response200
        return response

    def delete_incoming_webhook(
        self,
        hook_id: str,
    ) -> StatusOK:
        """Delete an incoming webhook

        Delete an incoming webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `DeleteIncomingWebhook <https://api.mattermost.com/#operation/DeleteIncomingWebhook>`_
        """

        url = f"/hooks/incoming/{hook_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_outgoing_webhooks(
        self,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
        team_id: Optional[str] = None,
        channel_id: Optional[str] = None,
    ) -> List[OutgoingWebhook]:
        """List outgoing webhooks

        Get a page of a list of outgoing webhooks. Optionally filter for a
        specific team or channel using query parameters.

        Permissions:
            `manage_webhooks` for the system or `manage_webhooks` for
            the specific team/channel.

        Api Reference:
            `GetOutgoingWebhooks <https://api.mattermost.com/#operation/GetOutgoingWebhooks>`_
        """

        url = "/hooks/outgoing"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "team_id": team_id,
            "channel_id": channel_id,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = OutgoingWebhook.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def create_outgoing_webhook(
        self,
        *,
        json_body: Union[CreateOutgoingWebhookJsonBody, Dict],
    ) -> OutgoingWebhook:
        """Create an outgoing webhook

        Create an outgoing webhook for a team.

        `manage_others_outgoing_webhooks` for the team the webhook is in if the
        user is different than the requester.

        Permissions:
            `manage_webhooks` for the team the webhook is in.

        Api Reference:
            `CreateOutgoingWebhook <https://api.mattermost.com/#operation/CreateOutgoingWebhook>`_
        """

        url = "/hooks/outgoing"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = OutgoingWebhook.parse_obj(response.json())

            return response201
        return response

    def get_outgoing_webhook(
        self,
        hook_id: str,
    ) -> OutgoingWebhook:
        """Get an outgoing webhook

        Get an outgoing webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `GetOutgoingWebhook <https://api.mattermost.com/#operation/GetOutgoingWebhook>`_
        """

        url = f"/hooks/outgoing/{hook_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = OutgoingWebhook.parse_obj(response.json())

            return response200
        return response

    def update_outgoing_webhook(
        self,
        hook_id: str,
        *,
        json_body: Union[UpdateOutgoingWebhookJsonBody, Dict],
    ) -> OutgoingWebhook:
        """Update an outgoing webhook

        Update an outgoing webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `UpdateOutgoingWebhook <https://api.mattermost.com/#operation/UpdateOutgoingWebhook>`_
        """

        url = f"/hooks/outgoing/{hook_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = OutgoingWebhook.parse_obj(response.json())

            return response200
        return response

    def delete_outgoing_webhook(
        self,
        hook_id: str,
    ) -> StatusOK:
        """Delete an outgoing webhook

        Delete an outgoing webhook given the hook id.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `DeleteOutgoingWebhook <https://api.mattermost.com/#operation/DeleteOutgoingWebhook>`_
        """

        url = f"/hooks/outgoing/{hook_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def regen_outgoing_hook_token(
        self,
        hook_id: str,
    ) -> StatusOK:
        """Regenerate the token for the outgoing webhook.

        Regenerate the token for the outgoing webhook.

        Permissions:
            `manage_webhooks` for system or `manage_webhooks` for the
            specific team or `manage_webhooks` for the channel.

        Api Reference:
            `RegenOutgoingHookToken <https://api.mattermost.com/#operation/RegenOutgoingHookToken>`_
        """

        url = f"/hooks/outgoing/{hook_id}/regen_token"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Module to access the Cluster endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import List

from ...models import ClusterInfo
from ..base import ApiBaseClass


class ClusterApi(ApiBaseClass):
    """Endpoints for configuring and interacting with high availability
    clusters."""

    def get_cluster_status(
        self,
    ) -> List[ClusterInfo]:
        """Get cluster status

        Get a set of information for each node in the cluster, useful for
        checking the status and health of each node.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `GetClusterStatus <https://api.mattermost.com/#operation/GetClusterStatus>`_
        """

        url = "/cluster/status"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = ClusterInfo.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response



""" Module to access the Bleve endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from ...models import StatusOK
from ..base import ApiBaseClass


class BleveApi(ApiBaseClass):
    """ """

    async def purge_bleve_indexes(
        self,
    ) -> StatusOK:
        """Purge all Bleve indexes

        Deletes all Bleve indexes and their contents. After calling this
        endpoint, it is
        necessary to schedule a new Bleve indexing job to repopulate the
        indexes.

        Permissions:
            Must have `sysconsole_write_experimental` permission.
        Minimum Server Version:
            5.24

        Api Reference:
            `PurgeBleveIndexes <https://api.mattermost.com/#operation/PurgeBleveIndexes>`_
        """

        url = "/bleve/purge_indexes"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Module to access the OpenGraph endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, Union

from pydantic import BaseModel

from ...models import OpenGraph, OpenGraphJsonBody
from ..base import ApiBaseClass


class OpenGraphApi(ApiBaseClass):
    """Endpoint for getting Open Graph metadata."""

    async def open_graph(
        self,
        *,
        json_body: Union[OpenGraphJsonBody, Dict],
    ) -> OpenGraph:
        """Get open graph metadata for url

        Get Open Graph Metadata for a specif URL. Use the Open Graph protocol to
        get some generic metadata about a URL. Used for creating link previews.

        Permissions:
            No permission required but must be logged in.
        Minimum Server Version:
            3.10

        Api Reference:
            `OpenGraph <https://api.mattermost.com/#operation/OpenGraph>`_
        """

        url = "/opengraph"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = OpenGraph.parse_obj(response.json())

            return response200
        return response



""" Module to access the Ldap endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    LDAPGroupsPaged,
    MigrateAuthToLdapJsonBody,
    MigrateIdLdapJsonBody,
    StatusOK,
    UploadLdapPrivateCertificateMultipartData,
    UploadLdapPublicCertificateMultipartData,
)
from ..base import ApiBaseClass


class LdapApi(ApiBaseClass):
    """Endpoints for configuring and interacting with LDAP."""

    def migrate_auth_to_ldap(
        self,
        *,
        json_body: Union[MigrateAuthToLdapJsonBody, Dict],
    ) -> None:
        """Migrate user accounts authentication type to LDAP.

        Migrates accounts from one authentication provider to another. For
        example, you can upgrade your authentication provider from email to
        LDAP.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.28

        Api Reference:
            `MigrateAuthToLdap <https://api.mattermost.com/#operation/MigrateAuthToLdap>`_
        """

        url = "/users/migrate_auth/ldap"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def sync_ldap(
        self,
    ) -> StatusOK:
        """Sync with LDAP

        Synchronize any user attribute changes in the configured AD/LDAP server
        with Mattermost.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `SyncLdap <https://api.mattermost.com/#operation/SyncLdap>`_
        """

        url = "/ldap/sync"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def test_ldap(
        self,
    ) -> StatusOK:
        """Test LDAP configuration

        Test the current AD/LDAP configuration to see if the AD/LDAP server can
        be contacted successfully.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `TestLdap <https://api.mattermost.com/#operation/TestLdap>`_
        """

        url = "/ldap/test"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def get_ldap_groups(
        self,
        *,
        q: Optional[str] = None,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[LDAPGroupsPaged]:
        """Returns a list of LDAP groups



        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.11

        Api Reference:
            `GetLdapGroups <https://api.mattermost.com/#operation/GetLdapGroups>`_
        """

        url = "/ldap/groups"
        params: Dict[str, Any] = {
            "q": q,
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = LDAPGroupsPaged.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def link_ldap_group(
        self,
        remote_id: str,
    ) -> StatusOK:
        """Link a LDAP group



        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.11

        Api Reference:
            `LinkLdapGroup <https://api.mattermost.com/#operation/LinkLdapGroup>`_
        """

        url = f"/ldap/groups/{remote_id}/link"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = StatusOK.parse_obj(response.json())

            return response201
        return response

    def migrate_id_ldap(
        self,
        *,
        json_body: Union[MigrateIdLdapJsonBody, Dict],
    ) -> StatusOK:
        """Migrate Id LDAP

        Migrate LDAP IdAttribute to new value.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.26

        Api Reference:
            `MigrateIdLdap <https://api.mattermost.com/#operation/MigrateIdLdap>`_
        """

        url = "/ldap/migrateid"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def upload_ldap_public_certificate(
        self,
        *,
        multipart_data: Union[UploadLdapPublicCertificateMultipartData, Dict],
    ) -> StatusOK:
        """Upload public certificate

        Upload the public certificate to be used for TLS verification. The
        server will pick a hard-coded filename for the PublicCertificateFile
        setting in your `config.json`.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `UploadLdapPublicCertificate <https://api.mattermost.com/#operation/UploadLdapPublicCertificate>`_
        """

        url = "/ldap/certificate/public"

        multipart_body_data = UploadLdapPublicCertificateMultipartData.parse_obj(
            multipart_data
        )

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_ldap_public_certificate(
        self,
    ) -> StatusOK:
        """Remove public certificate

        Delete the current public certificate being used for TLS verification.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `DeleteLdapPublicCertificate <https://api.mattermost.com/#operation/DeleteLdapPublicCertificate>`_
        """

        url = "/ldap/certificate/public"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def upload_ldap_private_certificate(
        self,
        *,
        multipart_data: Union[UploadLdapPrivateCertificateMultipartData, Dict],
    ) -> StatusOK:
        """Upload private key

        Upload the private key to be used for TLS verification. The server will
        pick a hard-coded filename for the PrivateKeyFile setting in your
        `config.json`.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `UploadLdapPrivateCertificate <https://api.mattermost.com/#operation/UploadLdapPrivateCertificate>`_
        """

        url = "/ldap/certificate/private"

        multipart_body_data = UploadLdapPrivateCertificateMultipartData.parse_obj(
            multipart_data
        )

        request_kwargs = {
            "url": url,
            "data": multipart_body_data.get_data(),
            "files": multipart_body_data.get_files(),
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def delete_ldap_private_certificate(
        self,
    ) -> StatusOK:
        """Remove private key

        Delete the current private key being used with your TLS verification.

        Permissions:
            Must have `manage_system` permission.

        Api Reference:
            `DeleteLdapPrivateCertificate <https://api.mattermost.com/#operation/DeleteLdapPrivateCertificate>`_
        """

        url = "/ldap/certificate/private"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response



""" Module to access the Threads endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, Optional

from ...models import UserThreads
from ..base import ApiBaseClass


class ThreadsApi(ApiBaseClass):
    """ """

    def get_user_threads(
        self,
        user_id: str,
        team_id: str,
        *,
        since: Optional[int] = None,
        deleted: Optional[bool] = False,
        extended: Optional[bool] = False,
        page: Optional[int] = 0,
        pageSize: Optional[int] = 30,
        totalsOnly: Optional[bool] = False,
    ) -> UserThreads:
        """Get all threads that user is following

        Get all threads that user is following

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `GetUserThreads <https://api.mattermost.com/#operation/GetUserThreads>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads"
        params: Dict[str, Any] = {
            "since": since,
            "deleted": deleted,
            "extended": extended,
            "page": page,
            "pageSize": pageSize,
            "totalsOnly": totalsOnly,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = UserThreads.parse_obj(response.json())

            return response200
        return response

    def get_thread_mention_counts_by_channel(
        self,
        user_id: str,
        team_id: str,
    ) -> None:
        """Get all unread mention counts from followed threads, per-channel

        Get all unread mention counts from followed threads

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `GetThreadMentionCountsByChannel <https://api.mattermost.com/#operation/GetThreadMentionCountsByChannel>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/mention_counts"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def update_threads_read_for_user(
        self,
        user_id: str,
        team_id: str,
    ) -> None:
        """Mark all threads that user is following as read

        Mark all threads that user is following as read

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `UpdateThreadsReadForUser <https://api.mattermost.com/#operation/UpdateThreadsReadForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/read"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def update_thread_read_for_user(
        self,
        user_id: str,
        team_id: str,
        thread_id: str,
        timestamp: str,
    ) -> None:
        """Mark a thread that user is following read state to the timestamp

        Mark a thread that user is following as read

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `UpdateThreadReadForUser <https://api.mattermost.com/#operation/UpdateThreadReadForUser>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/{thread_id}/read/{timestamp}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def start_following_thread(
        self,
        user_id: str,
        team_id: str,
        thread_id: str,
    ) -> None:
        """Start following a thread

        Start following a thread

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `StartFollowingThread <https://api.mattermost.com/#operation/StartFollowingThread>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/{thread_id}/following"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def stop_following_thread(
        self,
        user_id: str,
        team_id: str,
        thread_id: str,
    ) -> None:
        """Stop following a thread

        Stop following a thread

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `StopFollowingThread <https://api.mattermost.com/#operation/StopFollowingThread>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/{thread_id}/following"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response

    def get_user_thread(
        self,
        user_id: str,
        team_id: str,
        thread_id: str,
    ) -> None:
        """Get a thread followed by the user

        Get a thread

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission.
        Minimum Server Version:
            5.29

        Api Reference:
            `GetUserThread <https://api.mattermost.com/#operation/GetUserThread>`_
        """

        url = f"/users/{user_id}/teams/{team_id}/threads/{thread_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        return response



""" Module to access the Reactions endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Dict, List, Union

from pydantic import BaseModel

from ...models import PostIdToReactionsMap, Reaction, StatusOK
from ..base import ApiBaseClass


class ReactionsApi(ApiBaseClass):
    """Endpoints for creating, getting and removing emoji reactions."""

    async def save_reaction(
        self,
        *,
        json_body: Union[Reaction, Dict],
    ) -> Reaction:
        """Create a reaction

        Create a reaction.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in.

        Api Reference:
            `SaveReaction <https://api.mattermost.com/#operation/SaveReaction>`_
        """

        url = "/reactions"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Reaction.parse_obj(response.json())

            return response201
        return response

    async def get_reactions(
        self,
        post_id: str,
    ) -> List[Reaction]:
        """Get a list of reactions to a post

        Get a list of reactions made by all users to a given post.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in.

        Api Reference:
            `GetReactions <https://api.mattermost.com/#operation/GetReactions>`_
        """

        url = f"/posts/{post_id}/reactions"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Reaction.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def delete_reaction(
        self,
        user_id: str,
        post_id: str,
        emoji_name: str,
    ) -> StatusOK:
        """Remove a reaction from a post

        Deletes a reaction made by a user from the given post.

        Permissions:
            Must be user or have `manage_system` permission.

        Api Reference:
            `DeleteReaction <https://api.mattermost.com/#operation/DeleteReaction>`_
        """

        url = f"/users/{user_id}/posts/{post_id}/reactions/{emoji_name}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_bulk_reactions(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> PostIdToReactionsMap:
        """Bulk get the reaction for posts

        Get a list of reactions made by all users to a given post.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in.
        Minimum Server Version:
            5.8

        Api Reference:
            `GetBulkReactions <https://api.mattermost.com/#operation/GetBulkReactions>`_
        """

        url = "/posts/ids/reactions"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostIdToReactionsMap.parse_obj(response.json())

            return response200
        return response



""" Module to access the Posts endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    ChannelUnreadAt,
    CreatePostEphemeralJsonBody,
    CreatePostJsonBody,
    FileInfo,
    PatchPostJsonBody,
    Post,
    PostList,
    PostListWithSearchMatches,
    SearchPostsJsonBody,
    StatusOK,
    UpdatePostJsonBody,
)
from ..base import ApiBaseClass


class PostsApi(ApiBaseClass):
    """Endpoints for creating, getting and interacting with posts."""

    async def create_post(
        self,
        *,
        json_body: Union[CreatePostJsonBody, Dict],
        set_online: Optional[bool] = None,
    ) -> Post:
        """Create a post

        Create a new post in a channel. To create the post as a comment on
        another post, provide `root_id`.

        Permissions:
            Must have `create_post` permission for the channel the post
            is being created in.

        Api Reference:
            `CreatePost <https://api.mattermost.com/#operation/CreatePost>`_
        """

        url = "/posts"
        params: Dict[str, Any] = {
            "set_online": set_online,
        }
        params = {k: v for k, v in params.items() if v is not None}

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Post.parse_obj(response.json())

            return response201
        return response

    async def create_post_ephemeral(
        self,
        *,
        json_body: Union[CreatePostEphemeralJsonBody, Dict],
    ) -> Post:
        """Create a ephemeral post

        Create a new ephemeral post in a channel.

        Permissions:
            Must have `create_post_ephemeral` permission (currently only
            given to system admin)

        Api Reference:
            `CreatePostEphemeral <https://api.mattermost.com/#operation/CreatePostEphemeral>`_
        """

        url = "/posts/ephemeral"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Post.parse_obj(response.json())

            return response201
        return response

    async def get_post(
        self,
        post_id: str,
    ) -> Post:
        """Get a post

        Get a single post.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in or if the channel is public, have the
            `read_public_channels` permission for the team.

        Api Reference:
            `GetPost <https://api.mattermost.com/#operation/GetPost>`_
        """

        url = f"/posts/{post_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Post.parse_obj(response.json())

            return response200
        return response

    async def update_post(
        self,
        post_id: str,
        *,
        json_body: Union[UpdatePostJsonBody, Dict],
    ) -> Post:
        """Update a post

        Update a post. Only the fields listed below are updatable, omitted
        fields will be treated as blank.

        Permissions:
            Must have `edit_post` permission for the channel the post is
            in.

        Api Reference:
            `UpdatePost <https://api.mattermost.com/#operation/UpdatePost>`_
        """

        url = f"/posts/{post_id}"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Post.parse_obj(response.json())

            return response200
        return response

    async def delete_post(
        self,
        post_id: str,
    ) -> StatusOK:
        """Delete a post

        Soft deletes a post, by marking the post as deleted in the database.
        Soft deleted posts will not be returned in post queries.

        Permissions:
            Must be logged in as the user or have `delete_others_posts`
            permission.

        Api Reference:
            `DeletePost <https://api.mattermost.com/#operation/DeletePost>`_
        """

        url = f"/posts/{post_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def set_post_unread(
        self,
        user_id: str,
        post_id: str,
    ) -> ChannelUnreadAt:
        """Mark as unread from a post.

        Mark a channel as being unread from a given post.
        Must have `edit_other_users` permission if the user is not the one
        marking the post for himself.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in or if the channel is public, have the
            `read_public_channels` permission for the team.
        Minimum Server Version:
            5.18

        Api Reference:
            `SetPostUnread <https://api.mattermost.com/#operation/SetPostUnread>`_
        """

        url = f"/users/{user_id}/posts/{post_id}/set_unread"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = ChannelUnreadAt.parse_obj(response.json())

            return response200
        return response

    async def patch_post(
        self,
        post_id: str,
        *,
        json_body: Union[PatchPostJsonBody, Dict],
    ) -> Post:
        """Patch a post

        Partially update a post by providing only the fields you want to update.
        Omitted fields will not be updated. The fields that can be updated are
        defined in the request body, all other provided fields will be ignored.

        Permissions:
            Must have the `edit_post` permission.

        Api Reference:
            `PatchPost <https://api.mattermost.com/#operation/PatchPost>`_
        """

        url = f"/posts/{post_id}/patch"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Post.parse_obj(response.json())

            return response200
        return response

    async def get_post_thread(
        self,
        post_id: str,
    ) -> PostList:
        """Get a thread

        Get a post and the rest of the posts in the same thread.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in or if the channel is public, have the
            `read_public_channels` permission for the team.

        Api Reference:
            `GetPostThread <https://api.mattermost.com/#operation/GetPostThread>`_
        """

        url = f"/posts/{post_id}/thread"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostList.parse_obj(response.json())

            return response200
        return response

    async def get_flagged_posts_for_user(
        self,
        user_id: str,
        *,
        team_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[PostList]:
        """Get a list of flagged posts

        Get a page of flagged posts of a user provided user id string. Selects
        from a channel, team, or all flagged posts by a user. Will only return
        posts from channels in which the user is member.

        Permissions:
            Must be user or have `manage_system` permission.

        Api Reference:
            `GetFlaggedPostsForUser <https://api.mattermost.com/#operation/GetFlaggedPostsForUser>`_
        """

        url = f"/users/{user_id}/posts/flagged"
        params: Dict[str, Any] = {
            "team_id": team_id,
            "channel_id": channel_id,
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = PostList.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def get_file_infos_for_post(
        self,
        post_id: str,
    ) -> List[FileInfo]:
        """Get file info for post

        Gets a list of file information objects for the files attached to a
        post.

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in.

        Api Reference:
            `GetFileInfosForPost <https://api.mattermost.com/#operation/GetFileInfosForPost>`_
        """

        url = f"/posts/{post_id}/files/info"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = FileInfo.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    async def get_posts_for_channel(
        self,
        channel_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
        since: Optional[int] = None,
        before: Optional[str] = None,
        after: Optional[str] = None,
    ) -> PostList:
        """Get posts for a channel

        Get a page of posts in a channel. Use the query parameters to modify the
        behaviour of this endpoint. The parameter `since` must not be used with
        any of `before`, `after`, `page`, and `per_page` parameters.
        If `since` is used, it will always return all posts modified since that
        time, ordered by their create time limited till 1000. A caveat with this
        parameter is that there is no guarantee that the returned posts will be
        consecutive. It is left to the clients to maintain state and fill any
        missing holes in the post order.

        Permissions:
            Must have `read_channel` permission for the channel.

        Api Reference:
            `GetPostsForChannel <https://api.mattermost.com/#operation/GetPostsForChannel>`_
        """

        url = f"/channels/{channel_id}/posts"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
            "since": since,
            "before": before,
            "after": after,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostList.parse_obj(response.json())

            return response200
        return response

    async def get_posts_around_last_unread(
        self,
        user_id: str,
        channel_id: str,
        *,
        limit_before: Optional[int] = 60,
        limit_after: Optional[int] = 60,
        skipFetchThreads: Optional[bool] = False,
        collapsedThreads: Optional[bool] = False,
        collapsedThreadsExtended: Optional[bool] = False,
    ) -> PostList:
        """Get posts around oldest unread

        Get the oldest unread post in the channel for the given user as well as
        the posts around it. The returned list is sorted in descending order
        (most recent post first).

        Permissions:
            Must be logged in as the user or have `edit_other_users`
            permission, and must have `read_channel` permission for the
            channel.
        Minimum Server Version:
            5.14

        Api Reference:
            `GetPostsAroundLastUnread <https://api.mattermost.com/#operation/GetPostsAroundLastUnread>`_
        """

        url = f"/users/{user_id}/channels/{channel_id}/posts/unread"
        params: Dict[str, Any] = {
            "limit_before": limit_before,
            "limit_after": limit_after,
            "skipFetchThreads": skipFetchThreads,
            "collapsedThreads": collapsedThreads,
            "collapsedThreadsExtended": collapsedThreadsExtended,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostList.parse_obj(response.json())

            return response200
        return response

    async def search_posts(
        self,
        team_id: str,
        *,
        json_body: Union[SearchPostsJsonBody, Dict],
    ) -> PostListWithSearchMatches:
        """Search for team posts

        Search posts in the team and from the provided terms string.

        Permissions:
            Must be authenticated and have the `view_team` permission.

        Api Reference:
            `SearchPosts <https://api.mattermost.com/#operation/SearchPosts>`_
        """

        url = f"/teams/{team_id}/posts/search"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = PostListWithSearchMatches.parse_obj(response.json())

            return response200
        return response

    async def pin_post(
        self,
        post_id: str,
    ) -> StatusOK:
        """Pin a post to the channel

        Pin a post to a channel it is in based from the provided post id string.

        Permissions:
            Must be authenticated and have the `read_channel` permission
            to the channel the post is in.

        Api Reference:
            `PinPost <https://api.mattermost.com/#operation/PinPost>`_
        """

        url = f"/posts/{post_id}/pin"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def unpin_post(
        self,
        post_id: str,
    ) -> StatusOK:
        """Unpin a post to the channel

        Unpin a post to a channel it is in based from the provided post id
        string.

        Permissions:
            Must be authenticated and have the `read_channel` permission
            to the channel the post is in.

        Api Reference:
            `UnpinPost <https://api.mattermost.com/#operation/UnpinPost>`_
        """

        url = f"/posts/{post_id}/unpin"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def do_post_action(
        self,
        post_id: str,
        action_id: str,
    ) -> StatusOK:
        """Perform a post action

        Perform a post action, which allows users to interact with integrations
        through posts.

        Permissions:
            Must be authenticated and have the `read_channel` permission
            to the channel the post is in.

        Api Reference:
            `DoPostAction <https://api.mattermost.com/#operation/DoPostAction>`_
        """

        url = f"/posts/{post_id}/actions/{action_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    async def get_posts_by_ids(
        self,
        *,
        json_body: Union[List[str], Dict],
    ) -> List[Post]:
        """Get posts by a list of ids

        Fetch a list of posts based on the provided postIDs

        Permissions:
            Must have `read_channel` permission for the channel the post
            is in or if the channel is public, have the
            `read_public_channels` permission for the team.

        Api Reference:
            `GetPostsByIds <https://api.mattermost.com/#operation/GetPostsByIds>`_
        """

        url = "/posts/ids"
        json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        async with self.client._get_httpx_client() as httpx_client:
            response = await httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Post.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response



"""Definition of Exceptions and Errors thrown"""

from typing import Optional

from pydantic import BaseModel


class ExceptionBody(BaseModel):
    """Exception Body definition for errors returned by the API"""

    id: str
    message: str
    request_id: str
    status_code: int
    detailed_error: Optional[str]
    is_oauth: Optional[bool]


class ApiError(Exception):
    """
    Base for api errors
    """

    def __init__(self, message, error_details: ExceptionBody = None):
        super().__init__(message)
        self.details = error_details


class InvalidOrMissingParameters(ApiError):
    """
    Raised when mattermost returns a
    400 Invalid or missing parameters in URL or request body
    """


class NoAccessTokenProvided(ApiError):
    """
    Raised when mattermost returns a
    401 No access token provided
    """


class NotEnoughPermissions(ApiError):
    """
    Raised when mattermost returns a
    403 Do not have appropriate permissions
    """


class ResourceNotFound(ApiError):
    """
    Raised when mattermost returns a
    404 Resource not found
    """


class MethodNotAllowed(ApiError):
    """
    Raised when mattermost returns a
    405 Method Not Allowed
    """


class ContentTooLarge(ApiError):
    """
    Raised when mattermost returns a
    413 Content too large
    """


class TooManyRequests(ApiError):
    """
    Raised when mattermost returns a
    429 Too many requests
    """


class InternalServerError(ApiError):
    """
    Raised when mattermost returns a
    500 Internal Server Error
    """


class FeatureDisabled(ApiError):
    """
    Raised when mattermost returns a
    501 Feature is disabled
    """



from ..client.base import BaseClient


class ApiBaseClass:
    def __init__(self, client: BaseClient, skip_response_parsing: bool = False):
        self.client = client
        self.skip_response_parsing = skip_response_parsing



""" Module to access the Schemes endpoints """
# pylint: disable=too-many-lines,too-many-locals,too-many-public-methods,too-few-public-methods

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from ...models import (
    Channel,
    CreateSchemeJsonBody,
    PatchSchemeJsonBody,
    Scheme,
    StatusOK,
    Team,
)
from ..base import ApiBaseClass


class SchemesApi(ApiBaseClass):
    """Endpoints for creating, getting and updating and deleting schemes."""

    def get_schemes(
        self,
        *,
        scope: Optional[str] = "",
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Scheme]:
        """Get the schemes.

        Get a page of schemes. Use the query parameters to modify the behaviour
        of this endpoint.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.0

        Api Reference:
            `GetSchemes <https://api.mattermost.com/#operation/GetSchemes>`_
        """

        url = "/schemes"
        params: Dict[str, Any] = {
            "scope": scope,
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Scheme.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def create_scheme(
        self,
        *,
        json_body: Union[CreateSchemeJsonBody, Dict],
    ) -> Scheme:
        """Create a scheme

        Create a new scheme.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.0

        Api Reference:
            `CreateScheme <https://api.mattermost.com/#operation/CreateScheme>`_
        """

        url = "/schemes"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.post(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 201:
            response201 = Scheme.parse_obj(response.json())

            return response201
        return response

    def get_scheme(
        self,
        scheme_id: str,
    ) -> Scheme:
        """Get a scheme

        Get a scheme from the provided scheme id.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.0

        Api Reference:
            `GetScheme <https://api.mattermost.com/#operation/GetScheme>`_
        """

        url = f"/schemes/{scheme_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Scheme.parse_obj(response.json())

            return response200
        return response

    def delete_scheme(
        self,
        scheme_id: str,
    ) -> StatusOK:
        """Delete a scheme

        Soft deletes a scheme, by marking the scheme as deleted in the database.

        Permissions:
            Must have `manage_system` permission.
        Minimum Server Version:
            5.0

        Api Reference:
            `DeleteScheme <https://api.mattermost.com/#operation/DeleteScheme>`_
        """

        url = f"/schemes/{scheme_id}"

        request_kwargs = {
            "url": url,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.delete(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = StatusOK.parse_obj(response.json())

            return response200
        return response

    def patch_scheme(
        self,
        scheme_id: str,
        *,
        json_body: Union[PatchSchemeJsonBody, Dict],
    ) -> Scheme:
        """Patch a scheme

        Partially update a scheme by providing only the fields you want to
        update. Omitted fields will not be updated. The fields that can be
        updated are defined in the request body, all other provided fields will
        be ignored.

        Permissions:
            `manage_system` permission is required.
        Minimum Server Version:
            5.0

        Api Reference:
            `PatchScheme <https://api.mattermost.com/#operation/PatchScheme>`_
        """

        url = f"/schemes/{scheme_id}/patch"

        if isinstance(json_body, BaseModel):
            json_json_body = json_body.dict(exclude_unset=True)
        else:
            json_json_body = json_body

        request_kwargs = {
            "url": url,
            "json": json_json_body,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.put(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = Scheme.parse_obj(response.json())

            return response200
        return response

    def get_teams_for_scheme(
        self,
        scheme_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Team]:
        """Get a page of teams which use this scheme.

        Get a page of teams which use this scheme. The provided Scheme ID should
        be for a Team-scoped Scheme.
        Use the query parameters to modify the behaviour of this endpoint.

        Permissions:
            `manage_system` permission is required.
        Minimum Server Version:
            5.0

        Api Reference:
            `GetTeamsForScheme <https://api.mattermost.com/#operation/GetTeamsForScheme>`_
        """

        url = f"/schemes/{scheme_id}/teams"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Team.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response

    def get_channels_for_scheme(
        self,
        scheme_id: str,
        *,
        page: Optional[int] = 0,
        per_page: Optional[int] = 60,
    ) -> List[Channel]:
        """Get a page of channels which use this scheme.

        Get a page of channels which use this scheme. The provided Scheme ID
        should be for a Channel-scoped Scheme.
        Use the query parameters to modify the behaviour of this endpoint.

        Permissions:
            `manage_system` permission is required.
        Minimum Server Version:
            5.0

        Api Reference:
            `GetChannelsForScheme <https://api.mattermost.com/#operation/GetChannelsForScheme>`_
        """

        url = f"/schemes/{scheme_id}/channels"
        params: Dict[str, Any] = {
            "page": page,
            "per_page": per_page,
        }
        params = {k: v for k, v in params.items() if v is not None}

        request_kwargs = {
            "url": url,
            "params": params,
        }
        # pylint: disable-next=protected-access
        with self.client._get_httpx_client() as httpx_client:
            response = httpx_client.get(
                **request_kwargs,
            )

        if self.skip_response_parsing:
            return response

        if response.status_code == 200:
            response200 = []
            _response200 = response.json()
            for response200_item_data in _response200:
                response200_item = Channel.parse_obj(response200_item_data)

                response200.append(response200_item)

            return response200
        return response
